/*
    Project: TCTEMP
    Version: V1.17
    File   : na_fileread.c

    Copyright (c) 2006-2009 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.3 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "na_priv.h"

BOOL NAAPI NaReadFile(HANDLE hFile, PVOID pBuffer, ULONG ulLength,
  PULONG pulBytesRead, PLARGE_INTEGER pliByteOffset)
{
    NTSTATUS        ntStatus;
    IO_STATUS_BLOCK isb;

    NaAssert(pBuffer);

    ntStatus = NtReadFile(hFile, NULL, NULL, NULL,
      &isb, pBuffer, ulLength, pliByteOffset, NULL);
    if (ntStatus == STATUS_PENDING) {
        if (!NaWaitForFile(hFile, NULL)) {
            NaDbgPrintf(L"WaitForFile failed in ReadFile!");
            return FALSE;
        }
        ntStatus = isb.Status;
        if (!NT_SUCCESS(ntStatus)) {
            NaDbgPrintf(
              L"ReadFile failed after waiting (0x%08x)",
                ntStatus);
            return FALSE;
        }
    } else if (!NT_SUCCESS(ntStatus)) {
        NaDbgPrintf(L"ReadFile failed (0x%08x)", ntStatus);
        return FALSE;
    }

    if (pulBytesRead) {
        *pulBytesRead = (ULONG) isb.Information;
        return TRUE;
    }

    return (ULONG)isb.Information == ulLength;
}
